# -*- coding: utf-8 -*-
from .. import _
from plone.dexterity.browser.add import DefaultAddForm, DefaultAddView
from plone.dexterity.browser.edit import DefaultEditForm, DefaultEditView
from plone.dexterity.events import AddCancelledEvent
from plone.dexterity.events import EditCancelledEvent
from plone.dexterity.events import EditFinishedEvent
from plone.dexterity.i18n import MessageFactory as dmf
from Products.statusmessages.interfaces import IStatusMessage
from z3c.form import button
from z3c.form.interfaces import WidgetActionExecutionError
from zope.event import notify
from zope.interface import Invalid


class BaseForm(object):

    def additional_validation(self, data):
        # Some additional validation
        if (not data.get('remoteUrl') and not data.get('ISmartLinkExtension.internal_link')) or \
            bool(data.get('remoteUrl')) and bool(data.get('ISmartLinkExtension.internal_link')):
            raise WidgetActionExecutionError('remoteUrl',
                Invalid(_('error_internallink_externallink_doubled', default="You must select an internal link or enter an external link. You cannot have both."),))


class EditForm(BaseForm, DefaultEditForm):
    """
    """

    def updateWidgets(self):
        super(EditForm, self).updateWidgets()
        self.widgets['remoteUrl'].required = False
        self.fields['remoteUrl'].field.required = False

    @button.buttonAndHandler(dmf(u'Save'), name='save')
    def handleApply(self, action):
        data, errors = self.extractData()
        self.additional_validation(data)
        if errors:
            self.status = self.formErrorsMessage
            return
        self.applyChanges(data)
        IStatusMessage(self.request).addStatusMessage(
            dmf(u"Changes saved"), "info")
        self.request.response.redirect(self.nextURL())
        notify(EditFinishedEvent(self.context))

    @button.buttonAndHandler(dmf(u'Cancel'), name='cancel')
    def handleCancel(self, action):
        IStatusMessage(self.request).addStatusMessage(
            dmf(u"Edit cancelled"), "info")
        self.request.response.redirect(self.nextURL())
        notify(EditCancelledEvent(self.context))


class AddForm(BaseForm, DefaultAddForm):
    """
    """

    def updateWidgets(self):
        super(AddForm, self).updateWidgets()
        self.widgets['remoteUrl'].required = False
        self.fields['remoteUrl'].field.required = False

    @button.buttonAndHandler(dmf('Save'), name='save')
    def handleAdd(self, action):
        data, errors = self.extractData()
        self.additional_validation(data)
        if errors:
            self.status = self.formErrorsMessage
            return
        obj = self.createAndAdd(data)
        if obj is not None:
            # mark only as finished if we get the new object
            self._finishedAdd = True
            IStatusMessage(self.request).addStatusMessage(
                dmf(u"Item created"), "info")

    @button.buttonAndHandler(dmf(u'Cancel'), name='cancel')
    def handleCancel(self, action):
        IStatusMessage(self.request).addStatusMessage(
            dmf(u"Add New Item operation cancelled"), "info")
        self.request.response.redirect(self.nextURL())
        notify(AddCancelledEvent(self.context))


class AddView(DefaultAddView):
    form = AddForm


class EditView(DefaultEditView):
    form = EditForm



# -*- coding: utf-8 -*-
from collective.smartlink import _
from plone.app.contenttypes.interfaces import ILink
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.autoform import directives
from plone.namedfile.field import NamedBlobImage
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
@implementer(ILink)
class ISmartLinkExtension(model.Schema):

    # form.widget(internal_link=ContentTreeFieldWidget)
    internal_link = RelationChoice(
        title=_(u'label_smartlink_internallink', default='Internal link'),
        description=_(u'help_smartlink_internallink',
                      default=u"Browse to find the internal page to which you wish to link. "
                      u"If this field is used, then any entry in the external link field will be ignored. "
                      u"You cannot have both an internal and external link."),
        required=False,
        vocabulary='plone.app.vocabularies.Catalog'
    )
    directives.widget(
        'internal_link',
        RelatedItemsFieldWidget,
        vocabulary='plone.app.vocabularies.Catalog'
    )



# -*- coding: utf-8 -*-
from plone.app.contenttypes.browser.link_redirect_view import LinkRedirectView as Base
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from plone import api
from .. import _


class LinkRedirectView(Base):
    index = ViewPageTemplateFile('templates/link.pt')

    def _url_uses_scheme(self, schemes, url=None):
        url = url or self.context.remoteUrl
        if not url:
            return False
        for scheme in schemes:
            if url.startswith(scheme):
                return True
        return False

    def absolute_target_url(self):
        """Compute the absolute target URL."""
        context = self.context
        mtool = getToolByName(context, 'portal_membership')
        can_edit = mtool.checkPermission('Modify portal content', context)

        if self.context.internal_link:
            if not self.context.internal_link.to_object:
                if not can_edit:
		    api.portal.show_message(
				    message=_("link_not_found"),
				    request=self.request,
				    type='warning')
                    return self.request.response.redirect(api.portal.get().absolute_url())
                else:
                    return
            return self.context.internal_link.to_object.absolute_url()
        else:
            return super(LinkRedirectView, self).absolute_target_url()



# -*- coding: utf-8 -*-

from zope import interface
from Products.CMFCore.utils import getToolByName
from plone import api


def set_remote_url(object, event):
    """
    Set remote url for the link
    """
    #import pdb; pdb.set_trace()
    internal_link = getattr(object, 'internal_link', '')
    if internal_link:
        portal = api.portal.get()
        internal_link = '%s/%s' % (portal.absolute_url_path(), internal_link)
        #internal_link = '${navigation_root_url}/%s' % internal_link
        setattr(object, 'remoteUrl', internal_link)



# -*- coding: utf-8 -*-
from plone.app.contenttypes.utils import replace_link_variables_by_paths
from plone.indexer.decorator import indexer

from collective.smartlink.behaviors.interfaces import ISmartLinkExtension
from collective.smartlink import logger


@indexer(ISmartLinkExtension)
def getRemoteUrl(obj):
    if obj.internal_link and obj.remoteUrl and obj.remoteUrl != "http://":
        logger.warning(
            'for the %s link there are both internal:%s and external:%s url',
            obj.absolute_url(), obj.internal_link.to_path, obj.remoteUrl)
    if obj.internal_link:
        return obj.internal_link.to_path
    else:
        return replace_link_variables_by_paths(obj, obj.remoteUrl)
